/*
 * The MIT License (MIT)
 *
 * Copyright © 2023 xrv <xrv@live.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package com.tzld.crawler.etl.service.impl;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.google.common.base.Stopwatch;
import com.google.common.base.Strings;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Lists;
import com.tzld.commons.aliyun.oss.AliyunOssManager;
import com.tzld.crawler.etl.common.CustomValidator;
import com.tzld.crawler.etl.common.base.CommonResponse;
import com.tzld.crawler.etl.common.base.Constant;
import com.tzld.crawler.etl.common.enums.ExceptionEnum;
import com.tzld.crawler.etl.common.exception.CommonException;
import com.tzld.crawler.etl.dao.mapper.CrawlerUserV3Mapper;
import com.tzld.crawler.etl.dao.mapper.CrawlerVideoMapper;
import com.tzld.crawler.etl.dao.mapper.ext.CrawlerVideoExtMapper;
import com.tzld.crawler.etl.enums.MetricTypeEnum;
import com.tzld.crawler.etl.model.dto.MetricLogDto;
import com.tzld.crawler.etl.model.dto.StrategyDataDto;
import com.tzld.crawler.etl.model.dto.VideoInfoDto;
import com.tzld.crawler.etl.model.param.CrawlerVideoSendParam;
import com.tzld.crawler.etl.model.po.CrawlerUserV3;
import com.tzld.crawler.etl.model.po.CrawlerUserV3Example;
import com.tzld.crawler.etl.model.po.CrawlerVideo;
import com.tzld.crawler.etl.model.vo.CrawlerEtlParam;
import com.tzld.crawler.etl.model.vo.WxVideoVO;
import com.tzld.crawler.etl.service.EtlService;
import com.tzld.crawler.etl.service.SlsService;
import com.tzld.crawler.etl.service.feign.LongVideoFeign;
import com.tzld.crawler.etl.service.strategy.StrategyHandlerService;
import com.tzld.crawler.etl.util.*;
import org.apache.commons.lang3.tuple.Pair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.function.BooleanSupplier;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.function.Supplier;

/**
 * @author ehlxr
 * @since 2023-06-09 15:23.
 */
@Service
public class EtlServiceImpl implements EtlService {
    private static final Logger log = LoggerFactory.getLogger(EtlServiceImpl.class);
    private final StrategyHandlerService strategyHandlerService;
    private final AliyunOssManager aliyunOssManager;
    private final LongVideoFeign longVideoFeign;
    private final CrawlerVideoMapper crawlerVideoMapper;
    private final SlsService slsService;
    private final CrawlerVideoExtMapper crawlerVideoExtMapper;
    private final CrawlerUserV3Mapper crawlerUserV3Mapper;


    @Value("${download.file.path:/data/crawler/videos}")
    private String downloadPath;
    @Value("${aliyun.oss.video.bucket:art-pubbucket}")
    private String ossBucket;
    @Value("${env:}")
    private String env;
    @Value("${feishu.appid:}")
    private String feishuAppid;
    @Value("${feishu.appsecret:}")
    private String feishuAppsecret;
    @ApolloJsonValue("${feishu.sheet.token.map:{}}")
    private Map<String, String> feishuSheetTokenMap;
    @ApolloJsonValue("${feishu.sheet.id.map:{}}")
    private Map<String, String> feishuSheetIdMap;
    @ApolloJsonValue("${feishu.range.map:{}}")
    private Map<String, String> feishuRangeMap;
    @Value("${admin.cms.url:https://testadmin.piaoquantv.com/cms/post-detail/%d/info}")
    private String adminCmsUrl;
    @Value("${ffprobe.path:ffprobe}")
    private String ffprobePath;
    @Value("${ffprobe.path:ffmpeg}")
    private String ffmpegPath;
    @ApolloJsonValue("${fail.retry.times:{}}")
    private Map<String, Integer> failRetryTimes;
    // @Value("${youget.path:you-get}")
    // private String yougetPath;
    // @Value("${youget.download.platform:}")
    // private List<String> useYougetPlatfrom;

    @Value("${ua.platform:}")
    private List<String> randomUaPlatform;

    @Value("${proxy.platform:}")
    private List<String> proxyPlatform;

    @ApolloJsonValue("${proxy.info:{}}")
    private Map<String, String> proxyInfo;

    private Executor pool;

    private final ThreadLocal<CrawlerEtlParam> tlParam = new ThreadLocal<>();

    public EtlServiceImpl(StrategyHandlerService strategyHandlerService, AliyunOssManager aliyunOssManager,
                          LongVideoFeign longVideoFeign, CrawlerVideoMapper crawlerVideoMapper, SlsService slsService,
                          CrawlerVideoExtMapper crawlerVideoExtMapper, CrawlerUserV3Mapper crawlerUserV3Mapper) {
        this.strategyHandlerService = strategyHandlerService;
        this.aliyunOssManager = aliyunOssManager;
        this.longVideoFeign = longVideoFeign;
        this.crawlerVideoMapper = crawlerVideoMapper;
        this.slsService = slsService;
        this.crawlerVideoExtMapper = crawlerVideoExtMapper;
        this.crawlerUserV3Mapper = crawlerUserV3Mapper;
    }

    @Override
    public void deal(CrawlerEtlParam param) {
        tlParam.set(param);
        Stopwatch sw = Stopwatch.createStarted();
        CrawlerVideo crawlerVideo = new CrawlerVideo();
        BeanUtils.copyProperties(param, crawlerVideo);
        long id = 0L;
        try {
            // 参数校验
            CustomValidator.Result result = CustomValidator.validate(param);
            if (result.notSuccess()) {
                throw new CommonException(ExceptionEnum.PARAM_ERROR, "param validate failed." + result.getMessage());
            }

            // 保存数据库（去重校验）
            id = save2db(crawlerVideo);
            if (id <= 0) {
                throw new CommonException(ExceptionEnum.DATA_ERROR, "save2db failed, duplicated out video id.");
            }

            // 策略应用
            // TODO: 获取用户选择策略
            List<String> strategies = Lists.newArrayList("titleScore");

            StrategyDataDto data = strategyHandlerService.execute(strategies, param);
            if (data == null) {
                throw new CommonException(ExceptionEnum.DATA_ERROR, param + " filter by strategies " + strategies);
            }

            // 音频、视频文件下载、合成，上传 OSS、清理视频信息
            // 视频封面下载、上传 OSS、清理视频信息
            processVideo(data);

            // 视频发布
            long videoId = videoSend(data);

            // 更新数据库
            retrySupplier(() -> {
                crawlerVideo.setVideoId(videoId);
                crawlerVideo.setTitleScore(data.getTitleScore());
                return crawlerVideoMapper.updateByPrimaryKeySelective(crawlerVideo) > 0;
            }, "video2db", String.format("update video info [%s] to db", crawlerVideo));

            // 视频写入飞书
            // async2Feishu(data, videoId);

            metric(MetricTypeEnum.SUCCESS);
            metric(MetricTypeEnum.DURATION, sw.stop().elapsed().getSeconds());
        } catch (Exception e) {
            log.error("etl server deal {} failed.", param, e);
            // 回滚数据
            if (id > 0) {
                try {
                    int i = crawlerVideoMapper.deleteByPrimaryKey(id);
                    if (i <= 0) {
                        log.error("delete data from db failed of {}", id);
                    }
                } catch (Exception e1) {
                    log.error("delete data from db failed of {}", id, e1);
                }
            }

            metric(MetricTypeEnum.FAIL, CommonUtils.getRootCause(e).getMessage());
            if (!(e instanceof CommonException)) {
                metric(MetricTypeEnum.UNKNOWN_EXCEPTION, CommonUtils.getRootCause(e).getMessage());
            }
        } finally {
            tlParam.remove();
        }
    }

    private long save2db(CrawlerVideo crawlerVideo) {
        Long id = retrySupplierR(() -> {
            try {
                crawlerVideoExtMapper.insertSelectiveReturnId(crawlerVideo);
                slsService.log("message", "视频信息写入数据库成功", "crawler", crawlerVideo.getPlatform(), "mode",
                        crawlerVideo.getStrategy());
                return Pair.of(false, crawlerVideo.getId());
            } catch (DuplicateKeyException e) {
                // 根据站外视频 ID 唯一约束 key 做去重校验
                log.info("out video id {} of platform {} strategy {} has exist!", crawlerVideo.getOutVideoId(),
                        crawlerVideo.getPlatform(), crawlerVideo.getStrategy());
                metric(MetricTypeEnum.DUPLICATED);
                return Pair.of(false, 0L);
            }
        }, "video2db", String.format("save video info [%s] to db", crawlerVideo));
        return id == null ? 0 : id;
    }

    private long videoSend(StrategyDataDto data) {
        String platform = data.getPlatform();
        CrawlerVideoSendParam request = new CrawlerVideoSendParam();
        request.setLoginUid(data.getUserId());
        request.setAppType(888888);
        request.setTitle(data.getVideoTitle());
        request.setVideoPath(data.getVideoOssPath());
        request.setCoverImgPath(data.getCoverOssPath());
        request.setTotalTime(data.getDuration());
        request.setVersionCode(1);
        request.setViewStatus(1);
        request.setCrawlerSrcId(data.getOutVideoId());
        request.setCrawlerSrcCode(platform.toUpperCase());

        String publishTime = data.getPublishTime();
        if (Strings.isNullOrEmpty(publishTime)) {
            request.setCrawlerSrcPublishTimestamp(System.currentTimeMillis());
        } else {
            LocalDateTime localDateTime =
                    LocalDateTime.parse(publishTime, DateTimeFormatter.ofPattern(Constant.STANDARD_FORMAT));
            request.setCrawlerSrcPublishTimestamp(localDateTime.toInstant(ZoneOffset.of("+8")).toEpochMilli());
        }
        request.setCrawlerTaskTimestamp(System.currentTimeMillis());
        CommonResponse<WxVideoVO> response = longVideoFeign.crawlerVideoSend(request);
        log.info("crawler send video request [{}] response [{}]", request, response);
        if (!response.isSuccess() || response.getData() == null || response.getData().getId() == null) {
            metric(MetricTypeEnum.VIDEO_SEND_FAILED, response);
            throw new CommonException(ExceptionEnum.INVOKE_VIDEOAPI_ERROR,
                    "invoke crawler send video failed!" + response);
        }
        return response.getData().getId();
    }

    private void async2Feishu(StrategyDataDto data, long videoId) {
        String platform = data.getPlatform();
        String strategy = data.getStrategy();
        pool.execute(() -> {
            try {
                String sheetToken = feishuSheetTokenMap.get(platform);
                if (Strings.isNullOrEmpty(sheetToken)) {
                    log.error("feishu.sheet.token.map config of [{}] is null", platform);
                    return;
                }
                String strategyType = Strings.nullToEmpty(data.getStrategyType());
                String sheetId = feishuSheetIdMap.get(platform + strategyType);
                if (Strings.isNullOrEmpty(sheetToken)) {
                    log.error("feishu.sheet.id.map config of [{}] is null", platform);
                    return;
                }
                String range = feishuRangeMap.get(platform + strategyType);

                log.info("platform [{}] strategy [{}] sheetToken is [{}], sheetId is [{}], range is [{}]", platform,
                        strategy, sheetToken, sheetId, range);
                String fsResp = FeishuUtils.insertRows(feishuAppid, feishuAppsecret, sheetToken, sheetId, 1, 2);
                log.debug("insert columns to feishu sheet response is [{}]", fsResp);

                List<List<Object>> values = new ArrayList<>();
                List<Object> value = new ArrayList<>();
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern(Constant.STANDARD_FORMAT);
                if ("search".equals(strategy)) {
                    CrawlerUserV3Example example = new CrawlerUserV3Example();
                    example.createCriteria().andUidEqualTo(data.getUserId());
                    List<CrawlerUserV3> crawlerUserV3s = crawlerUserV3Mapper.selectByExample(example);
                    value.add(!crawlerUserV3s.isEmpty() ? crawlerUserV3s.get(0).getLink() : "");
                }

                value.add(data.getTitleScore());
                value.add(videoId);
                value.add(formatter.format(LocalDateTime.ofInstant(Instant.ofEpochMilli(System.currentTimeMillis()),
                        ZoneId.systemDefault())));
                value.add(strategy);
                value.add(data.getOutVideoId());
                value.add(data.getVideoTitle());
                value.add(String.format(adminCmsUrl, videoId));
                value.add(data.getPlayCnt());
                value.add(data.getCommentCnt());
                value.add(data.getLikeCnt());
                value.add(data.getShareCnt());
                value.add(data.getDuration());
                value.add(data.getWidth() + "*" + data.getHeight());
                value.add(data.getPublishTime());
                value.add(data.getUserName());
                value.add(data.getOutUserId());
                value.add(data.getAvatarUrl());
                value.add(data.getCoverUrl());
                value.add(data.getVideoUrl());
                values.add(value);

                fsResp = FeishuUtils.updateValues(feishuAppid, feishuAppsecret, sheetToken, range, values);
                log.debug("update feishu sheet value response is [{}]", fsResp);

                slsService.log("message", "视频已保存至云文档", "crawler", platform, "mode", strategy);
            } catch (Exception e) {
                log.error("save data to feishu sheet error. platform [{}], strategy [{}]", platform, strategy, e);
            }
        });
    }

    private void processVideo(StrategyDataDto data) throws IOException {
        String title = data.getVideoTitle();
        String platform = data.getPlatform();
        String strategy = data.getStrategy();
        String audioUrl = data.getAudioUrl();

        String videoPath = urlDownload(data.getVideoUrl(), "longvideo/crawler_local/video", title, data.getOutVideoId(), platform);
        // 音、视频合成
        if (!Strings.isNullOrEmpty(audioUrl)) {
            String audioPath = urlDownload(data.getAudioUrl(), "longvideo/crawler_local/audio", title, data.getOutVideoId(), platform);
            try {
                retryFunc(t -> {
                    try {
                        VideoUtils.videoSynthesis(ffmpegPath, downloadPath + File.separator + t,
                                downloadPath + File.separator + audioPath, downloadPath + File.separator + t + "_comp.mp4");
                        return false;
                    } catch (Exception e) {
                        throw new RuntimeException(e);
                    }
                }, videoPath, "synthesis", String.format("video [%s] audio [%s] synthesis", videoPath, audioPath));
            } catch (Exception e) {
                metric(MetricTypeEnum.VIDEO_SYNTHESIS_FAILED, CommonUtils.getRootCause(e).getMessage());
                throw new CommonException(e);
            }

            // 清理合成音频之前的文件
            Files.deleteIfExists(Paths.get(new File(downloadPath + File.separator + videoPath).getPath()));
            videoPath += "_comp.mp4";
        }

        // 视频上传 oss
        String tempFilePath = downloadPath + File.separator + videoPath;
        file2oss(tempFilePath, videoPath, platform, strategy);
        data.setVideoOssPath(videoPath);
        if (data.getDuration() == null || data.getDuration() <= 0 || data.getWidth() == null || data.getWidth() <= 0
                || data.getHeight() == null || data.getHeight() <= 0) {
            // 获取视频时长、宽、高
            VideoInfoDto videoInfo = VideoUtils.getVideoInfo(tempFilePath, ffprobePath);
            if (videoInfo != null) {
                data.setDuration(videoInfo.getDuration());
                data.setWidth(videoInfo.getWidth());
                data.setHeight(videoInfo.getHeight());
            }
        }

        // 文件清理
        log.info("begin delete temp video file [{}]", tempFilePath);
        Files.deleteIfExists(Paths.get(new File(tempFilePath).getPath()));

        // 视频封面下载、上传 OSS
        String coverPath = urlDownload(data.getCoverUrl(), "longvideo/crawler_local/image", title, data.getOutVideoId(), platform);
        file2oss(downloadPath + File.separator + coverPath, coverPath, platform, strategy, "image/jpg");
        data.setCoverOssPath(coverPath);
        tempFilePath = downloadPath + File.separator + coverPath;

        // 文件清理
        log.info("begin delete temp cover file [{}]", tempFilePath);
        Files.deleteIfExists(Paths.get(new File(tempFilePath).getPath()));
    }

    private String urlDownload(String fileUrl, String typeDir, String title, String outVideoId, String platform) {
        String fileName = MD5Util.md5(title) + outVideoId;
        String relFileDir = typeDir + File.separator
                + env + File.separator
                + LocalDate.now().format(DateTimeFormatter.BASIC_ISO_DATE);
        String absFileDir = downloadPath + File.separator + relFileDir;

        File absFile = new File(absFileDir);
        if (!absFile.exists() && (!absFile.mkdirs())) {
            log.error("mkdir typeDir [{}] failed!", absFileDir);

            absFileDir = "/data";
        }

        String filePath = absFileDir + File.separator + fileName;
        try {
            retryFunc((param) -> {
                        String fpath = param.getOrDefault("filePath", "");
                        String platfrm = param.getOrDefault("platform", "");
                        try {
                            // 公众号系列域名
                            if (fileUrl.contains("mpvideo.qpic.cn")) {
                                FileUtils.downloadForGZH(fileUrl, fpath,
                                        !CollectionUtils.isEmpty(randomUaPlatform) && randomUaPlatform.contains(platfrm)
                                        , !CollectionUtils.isEmpty(proxyPlatform) && proxyPlatform.contains(platfrm),
                                        proxyInfo);
                            }
                            // 西瓜视频系列 1
                            else if (fileUrl.contains("v9-xg-web-pc.ixigua.com")) {
                                FileUtils.downloadForXGV9(fileUrl, fpath,
                                        !CollectionUtils.isEmpty(randomUaPlatform) && randomUaPlatform.contains(platfrm)
                                        , !CollectionUtils.isEmpty(proxyPlatform) && proxyPlatform.contains(platfrm),
                                        proxyInfo);
                            }
                            // 西瓜视频系列 2
                            else if (fileUrl.contains("v3-xg-web-pc.ixigua.com")) {
                                FileUtils.downloadForXGV3(fileUrl, fpath,
                                        !CollectionUtils.isEmpty(randomUaPlatform) && randomUaPlatform.contains(platfrm)
                                        , !CollectionUtils.isEmpty(proxyPlatform) && proxyPlatform.contains(platfrm),
                                        proxyInfo);
                            }
                            // 百度 cdn 系列域名
                            else if (fileUrl.contains("video-bos.cdn.bcebos.com")) {
                                FileUtils.downloadForFQW(fileUrl, fpath,
                                        !CollectionUtils.isEmpty(randomUaPlatform) && randomUaPlatform.contains(platfrm)
                                        , !CollectionUtils.isEmpty(proxyPlatform) && proxyPlatform.contains(platfrm),
                                        proxyInfo);
                            }
                            // 抖音
                            else if (platform.equals("douyin")) {
                                FileUtils.downloadDY(fileUrl, fpath,
                                        !CollectionUtils.isEmpty(randomUaPlatform) && randomUaPlatform.contains(platfrm)
                                        , !CollectionUtils.isEmpty(proxyPlatform) && proxyPlatform.contains(platfrm),
                                        proxyInfo);
                            }
                            else {
                                String tempUrl = fileUrl;
                                if (platform.equals("zhufuzhonglaonianrenruyijixiang")) {
                                    tempUrl = tempUrl.replace("https://api-hl.huoshan.com", "http://api-hl.huoshan.com");
                                }
                                FileUtils.download(tempUrl, fpath,
                                        !CollectionUtils.isEmpty(randomUaPlatform) && randomUaPlatform.contains(platfrm)
                                        , !CollectionUtils.isEmpty(proxyPlatform) && proxyPlatform.contains(platfrm),
                                        proxyInfo);
                            }
                            return false;
                        } catch (CommonException e) {
                            if (e.getCode() == ExceptionEnum.URL_FORBIDDEN.getCode()) {
                                log.error("access to the url [{}] of remote server is prohibited.", fileUrl);
                                metric(MetricTypeEnum.INVALID_URL, fileUrl);
                                return false;
                            }
                            throw new RuntimeException(e);
                        } catch (Exception e) {
                            throw new RuntimeException(e);
                        }
                    }, ImmutableMap.of("filePath", filePath, "platform", platform),
                    "download", String.format("download file from [%s] to [%s].", fileUrl, filePath));
        } catch (Exception e) {
            metric(MetricTypeEnum.DOWNLOAD_FAILED, String.format("download file from [%s] error. 'cause of [%s]",
                    fileUrl, CommonUtils.getRootCause(e).getMessage()));
            throw new CommonException(e);
        }

        File file = new File(filePath);
        if (!file.exists() || file.length() <= 0) {
            throw new CommonException(ExceptionEnum.DATA_ERROR, String.format("download file from [%s] is empty.", fileUrl));
        }

        return relFileDir + File.separator + fileName;
    }

    private void file2oss(String localFile, String ossBucketKey, String crawler, String mode, String contentType) {
        // 文件上传 OSS
        slsService.log("message", "开始上传视频... ", "crawler", crawler, "mode", mode);
        log.info("begin upload {} to oss key {}", localFile, ossBucketKey);

        retrySupplier(() -> {
            try {
                if (!Strings.isNullOrEmpty(contentType)) {
                    aliyunOssManager.putObject(ossBucket, ossBucketKey, Files.newInputStream(Paths.get(localFile)), contentType);
                } else {
                    aliyunOssManager.putObject(ossBucket, ossBucketKey, Files.newInputStream(Paths.get(localFile)));
                }
                return false;
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }, "2oss", String.format("upload file [%s] to oss [%s]", localFile, ossBucketKey));
    }

    private void file2oss(String localFile, String ossBucketKey, String crawler, String mode) {
        file2oss(localFile, ossBucketKey, crawler, mode, null);
    }

    private void retrySupplier(BooleanSupplier supplier, String type, String errorMsg) {
        retryFunc(c -> supplier.getAsBoolean(), null, type, errorMsg);
    }

    private <R> R retrySupplierR(Supplier<Pair<Boolean, R>> supplier, String type, String errorMsg) {
        return retryFuncR(c -> supplier.get(), null, type, errorMsg);
    }

    private <T, R> R retryFuncR(Function<T, Pair<Boolean, R>> func, T t, String type, String errorMsg) {
        int retry = 0;
        int retryTimes = failRetryTimes.getOrDefault(type, 3);
        R r = null;
        while (retry < retryTimes) {
            retry++;
            try {
                Pair<Boolean, R> apply = func.apply(t);
                r = apply.getRight();
                if (Boolean.FALSE.equals(apply.getLeft())) {
                    break;
                }
            } catch (Exception e) {
                log.error("the operation '{}' has failed on the {}th retry.", errorMsg, retry, e);
                if (retry >= retryTimes) {
                    throw new CommonException(ExceptionEnum.SYSTEM_ERROR,
                            "the operation '" + errorMsg + "' has failed after " + retry + " times retry. 'cause of [" + e.getMessage() + "]");
                }
            }
        }
        return r;
    }

    private <T> void retryFunc(Predicate<T> func, T t, String type, String errorMsg) {
        retryFuncR(c -> Pair.of(func.test(c), null), t, type, errorMsg);
    }

    private void metric(MetricTypeEnum typeEnum, Object data) {
        CrawlerEtlParam param = tlParam.get();
        slsService.metric(MetricLogDto.newBuiler()
                .platform(param.getPlatform())
                .strategy(param.getStrategy())
                .type(typeEnum)
                .data(data)
                .msg(param.toString()).build());
    }

    private void metric(MetricTypeEnum typeEnum) {
        metric(typeEnum, "");
    }

    @PostConstruct
    public void init() {
        pool = Executors.newFixedThreadPool(1);
    }
}
