/*
 * The MIT License (MIT)
 *
 * Copyright © 2023 xrv <xrv@live.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package com.tzld.crawler.etl.service.strategy;

import com.tzld.crawler.etl.model.dto.StrategyDataDto;
import com.tzld.crawler.etl.model.vo.CrawlerEtlParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author ehlxr
 * @since 2023-06-09 15:06.
 */
@Service
public class StrategyHandlerService implements ApplicationContextAware {
    private static final Logger log = LoggerFactory.getLogger(StrategyHandlerService.class);
    private final Map<String, StrategyAbstractHandler> HANDLER_MAP = new HashMap<>();

    private ApplicationContext applicationContext;

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }

    @PostConstruct
    public void init() {
        // 获取 AbstractHandler 类的所有实现类
        Map<String, StrategyAbstractHandler> type = applicationContext.getBeansOfType(StrategyAbstractHandler.class);
        for (Map.Entry<String, StrategyAbstractHandler> entry : type.entrySet()) {
            StrategyAbstractHandler value = entry.getValue();
            log.info("load strategy {}", value.eventTypes());
            HANDLER_MAP.put(value.eventTypes(), value);
        }
    }

    /**
     * 业务处理类的路由方法
     */
    public StrategyDataDto execute(List<String> strategies, CrawlerEtlParam param) {
        StrategyDataDto result = null;
        for (String strategy : strategies) {
            StrategyAbstractHandler handler = HANDLER_MAP.get(strategy);
            if (handler == null) {
                log.error("can't find handler of {}", strategy);
                return null;
            }
            result = handler.execute(param);
            param = result;
        }
        return result;
    }
}
