/*
 * The MIT License (MIT)
 *
 * Copyright © 2023 xrv <xrv@live.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package com.tzld.crawler.etl.service.strategy.handler;

import com.alibaba.fastjson.JSONArray;
import com.huaban.analysis.jieba.JiebaSegmenter;
import com.huaban.analysis.jieba.SegToken;
import com.tzld.crawler.etl.model.dto.StrategyDataDto;
import com.tzld.crawler.etl.model.vo.CrawlerEtlParam;
import com.tzld.crawler.etl.service.strategy.StrategyAbstractHandler;
import com.tzld.crawler.etl.util.FeishuUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * @author ehlxr
 * @since 2023-06-19 17:41.
 */
@Component
public class TitleScoreHandler extends StrategyAbstractHandler {
    private static final Logger log = LoggerFactory.getLogger(TitleScoreHandler.class);

    @Value("${feishu.appid:}")
    private String feishuAppid;
    @Value("${feishu.appsecret:}")
    private String feishuAppsecret;
    @Value(("${title.stop.word.feishu.sheetId:}"))
    private String stopWordSheetId;
    @Value(("${title.stop.word.feishu.sheetToken:}"))
    private String stopWordSheetToken;
    @Value(("${title.stop.word.feishu.range:}"))
    private String stopWordSheetRange;
    @Value(("${title.score.word.feishu.sheetId:}"))
    private String scoreWordSheetId;
    @Value(("${title.score.word.feishu.sheetToken:}"))
    private String scoreWordSheetToken;
    @Value(("${title.score.word.feishu.range:}"))
    private String scoreWordSheetRange;

    private Map<String, Double> scoreWordMap = new HashMap<>();
    private List<String> stopWorlds = new ArrayList<>();

    @Override
    public StrategyDataDto execute(CrawlerEtlParam param) {
        double score = -1;
        StrategyDataDto result = new StrategyDataDto();
        BeanUtils.copyProperties(param, result);
        try {
            JiebaSegmenter segmenter = new JiebaSegmenter();
            List<SegToken> process = segmenter.process(param.getVideoTitle(), JiebaSegmenter.SegMode.SEARCH);
            for (SegToken segToken : process) {
                String word = segToken.word;
                if (stopWorlds.contains(word)) {
                    continue;
                }
                score += scoreWordMap.getOrDefault(word, 0.0);
            }
// workaround:低于0.3先不过滤，写入飞书后，运营人工处理
//        if (score <= 0.3) {
//            log.warn("title score is less than 0.3 {}", param);
//            slsService.log("mode", param.getStrategy(), "crawler", param.getPlatform(), "message", "权重分:" + score + "<=0.3");
//            return null;
//        }
        } catch (BeansException e) {
            log.error("failed to calculate title weight score! title [{}]", param.getVideoTitle(), e);
        }

        result.setTitleScore(score);
        return result;
    }

    @Override
    public String eventTypes() {
        return "titleScore";
    }

    @PostConstruct
    public void init() {
        ScheduledExecutorService executor = Executors.newSingleThreadScheduledExecutor();
        executor.scheduleWithFixedDelay(() -> {
            try {
                JSONArray values = FeishuUtils.getValues(feishuAppid, feishuAppsecret, stopWordSheetToken, stopWordSheetId, stopWordSheetRange);
                stopWorlds = new ArrayList<>();
                for (int i = 0; i < values.size(); i++) {
                    JSONArray row = values.getJSONArray(i);
                    String val = row.getString(0);
                    if (val != null) {
                        stopWorlds.add(val);
                    }
                }
                log.info("load stop word list size {}", stopWorlds.size());

                values = FeishuUtils.getValues(feishuAppid, feishuAppsecret, scoreWordSheetToken, scoreWordSheetId, scoreWordSheetRange);
                scoreWordMap = new HashMap<>();
                for (int i = 0; i < values.size(); i++) {
                    JSONArray row = values.getJSONArray(i);
                    String val = row.getString(0);
                    if (val != null) {
                        scoreWordMap.put(val, row.getDouble(8));
                    }
                }
                log.info("load score word map size {}", scoreWordMap.size());
            } catch (Exception e) {
                log.error("load stop/score word error.", e);
            }
        }, 0, 10, TimeUnit.MINUTES);
    }
}
