/*
 * The MIT License (MIT)
 *
 * Copyright © 2023 xrv <xrv@live.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package com.tzld.crawler.etl.util;

import com.tzld.crawler.etl.common.enums.ExceptionEnum;
import com.tzld.crawler.etl.common.exception.CommonException;
import com.tzld.crawler.etl.model.dto.VideoInfoDto;
import net.bramp.ffmpeg.FFprobe;
import net.bramp.ffmpeg.probe.FFmpegProbeResult;
import net.bramp.ffmpeg.probe.FFmpegStream;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.List;

/**
 * @author ehlxr
 * @since 2023-06-26 18:01.
 */
public class VideoUtils {
    private static final Logger log = LoggerFactory.getLogger(VideoUtils.class);

    public static VideoInfoDto getVideoInfo(String videoPath, String ffprobePath) {
        try {
            FFprobe ffprobe = new FFprobe(ffprobePath);
            FFmpegProbeResult probeResult = ffprobe.probe(videoPath);

            List<FFmpegStream> streams = probeResult.getStreams();
            int height = 0;
            int width = 0;
            for (FFmpegStream stream : streams) {
                height = height <= 0 ? stream.height : height;
                width = width <= 0 ? stream.width : width;
            }

            return VideoInfoDto.newBuilder()
                    .duration((long) probeResult.getFormat().duration)
                    .height(height)
                    .width(width)
                    .build();
        } catch (Exception e) {
            log.error("get video {} info by ffprobe error.", videoPath, e);
            return null;
        }
    }

    public static void videoSynthesis(String ffmpegPath, String videoPath, String audioPath, String outPath) throws Exception {
        // try {
        // FFmpeg fFmpeg = new FFmpeg("ffmpeg");
        // FFprobe fFprobe = new FFprobe("ffprobe");
        //
        // FFmpegBuilder builder = new FFmpegBuilder()
        //         .addInput(videoPath)
        //         // .addExtraArgs("-i", videoPath)  // 然后添加音频文件作为参数
        //         .addExtraArgs("-i", audioPath)  // 然后添加音频文件作为参数
        //         .addOutput(outPath)   // 输出文件
        //         .addExtraArgs("-c:v", "copy")  // 使用原视频的编码
        //         .addExtraArgs("-c:a", "aac")  // 音频用aac编码
        //         .addExtraArgs("-strict", "experimental")
        //         .addExtraArgs("-map", "0:v:0")  //  映射视频流
        //         .addExtraArgs("-map", "1:a:0")  //  映射音频流
        //         .done();
        // FFmpegExecutor executor = new FFmpegExecutor(fFmpeg, fFprobe);
        // executor.createJob(builder).run(); // 执行 ffmpeg 命令
        ProcessBuilder processBuilder = new ProcessBuilder(ffmpegPath, "-i", videoPath, "-i", audioPath,
                "-c:v", "copy", "-c:a", "aac", "-strict", "experimental", "-map", "0:v:0", "-map", "1:a:0", outPath);
        Process process = processBuilder.start();
        process.waitFor();

        File file = new File(outPath);
        if (!file.exists() || file.length() <= 0) {
            throw new CommonException(ExceptionEnum.DATA_ERROR, "video synthesis failed, No synthesized video file output.");
        }
        // } catch (Exception e) {
        //     log.error("video {} audio {} synthesis error.", videoPath, audioPath, e);
        //     throw new CommonException(ExceptionEnum.DATA_ERROR, "video synthesis error");
        // }
    }


    public static void main(String[] args) throws Exception {
        videoSynthesis("ffmpeg", "/datalog/crawler/videos/longvideo/crawler_local/video/dev/20230628/7628fedb3f1abf460733da698c7fc169",
                "/datalog/crawler/videos/longvideo/crawler_local/audit/dev/20230628/7628fedb3f1abf460733da698c7fc169",
                "/datalog/crawler/videos/longvideo/crawler_local/video/dev/20230628/34b192c574330bbcaf008b3c90e565b9_comp111100000");
    }
}
